/*  bunpf.c  */
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include "macro.h"    //18/12/8

extern FILE *prcs, *dbg;
extern int wnb, wc;
extern char cj[WordNumb][JSentLen], cz[][ZokuLen], md[][JSentLen],
	sj[], oj[], vr[], sz[], oz[],vz[], Dbg, LOG;
char en[WordNumb][SentLen];
static char op = 0x28, cp = 0x29;
static char 	bungo[14][18] = {"ł","ł( 2)","(ł 3)",
	"Ȃ( 3)","͂Ȃ","Ȃ","͂Ȃ( 3)",
	"͖( 3)","Ȃ( 3)","( 3)",
	"","( 2)","낤","ɂȂ( 3)"},
 	kougo[14][20] = {"ł","ł","ł","͂܂ł",
	"͂܂","܂","͂܂ł","͗L܂ł",
	"܂ł","L܂ł","܂","܂",
	"ł傤","ɂ܂ł"},
 	mzn[9][3] = {"","","","","","","","",""},
 	rny[9][3] = {"","","","","","","","",""},
	shs[9][3] = {"","","","","","","","",""},
        mei[9][3] = {"","","","","","","","",""},
        kanyu[9][5] = {"","","","Ƃ","̂","","낤",
	"","ڂ"},
	onb[9][5] = {"","","","","","","",
	"",""},
	kak[9][5] = {"","","","","","","",
	"",""},
	ksm[32][5] = {"","˂","","","","o","",
	"ς","Q","","f","","","","","",
	"","","Ă","ɂ","˂","݂","߂","","",
	"","","","","ł","т","ׂ"};

past(yk)
char yk[];
{
    int  k, j, ln;
    char *gobi, pars[JWordLen], daro[7];
    ln = strlen(yk);
    if(yk[ln-1] == ')') return;
    if(j = paren(yk)){
    }
    else *pars = '\0';
    if(ln > 5 && strcmp(yk+ln-6, "낤") == 0){
      strcpy(daro, "낤");	*(yk+ln-6) = '\0';	ln -= 6;
    }
    else *daro = '\0';
    gobi = yk+ln-2;
    if(strcmp(gobi, "") == 0 || strcmp(gobi, "") == 0){
      strcpy(gobi, "ł( 2)");	goto LAST2;
    }
    else {
      if(strcmp(gobi, "") == 0){
	strcpy(gobi, "( 3)");	goto LAST2;
      }
      if(strcmp(gobi, "I") == 0){
	strcat(gobi, "ł( 2)");	goto LAST2;
      }
      if(ln < 4) goto LAST2;
      gobi -= 2;
      if(strcmp(gobi, "") == 0){
	strcpy(gobi, "( 2)");	goto LAST2;
      }
      if(strcmp(gobi, "") == 0){
	strcpy(gobi, "( 2)");	goto LAST2;
      }
      if(strcmp(gobi, "s") == 0){
	strcpy(gobi, "s( 2)");	goto LAST2;
      }
      for(k = 0; k <= 31; k++)
	if(strcmp(gobi, ksm[k]) == 0){
	  strcpy(gobi+2, "( 1)");	goto LAST2;
	}
      gobi += 2;
      for(k = 0; k <= 8; k++)
	if(strcmp(gobi, shs[k]) == 0){ strcat(yk, " 2)");
	  ptinsert(gobi, "(");	ptinsert(gobi, kak[k]);	break;
	}
    }
    LAST2:	strcat(yk, daro);	strcat(yk, pars);
}

genkei(yk)
char yk[];
{
    int  ln, d;
    char *gobi, pars[JWordLen], *qt;
    *pars = '\0';
    ln = strlen(yk);
    gobi = yk+ln-4;
    if(strcmp(gobi, "") == 0){
      strcpy(gobi, "");	goto RTN;
    }
    gobi = yk+ln-2;
    if(strcmp(gobi, "") == 0 || strcmp(gobi, "") == 0){
      if(strcmp(yk+ln-6, "傫") == 0) { strcpy(gobi, "");    goto RTN; }
      strcpy(gobi, "ł");	goto RTN;
    }
    else{
      if(strcmp(gobi, "") == 0) goto RTN;
    }
    if(yk[ln-1] != ')'){
      if(qt = (char*)strstr(yk, " (")){
	if(*(qt-1) != ')') goto RTN;
	strcpy(pars, qt);	*qt = '\0';
      }
    }
    if(qt = (char*)index(yk, '(')){
      ln = strlen(yk);	d = yk[ln-2]-'0';
      *(yk+ln-3) = '\0';	ptptcut(qt-(d*2), qt);
    }
    RTN:	if(*pars) strcat(yk, pars);
}

paren(yk)
char yk[];
{
    int  kk, ln, j = 0;
    char Er, *dums, *pt1, *pt2;
    ln = strlen(yk);
    if(yk[ln-1] == cp){
      for(kk = ln-2; kk >= 0; kk--)
	if(yk[kk] == op){ j = kk+1;	return(j); }
      Er = '\0';	puts("( ȂB@G[");
      while(! Er){
	puts("̃L[ĉB");	Er = getchar();
      }
    }
    dums = yk+ln-2;
    if(strcmp(dums, "j") == 0){
      pt1 = yk;
      while(pt2 = (char*)strstr(pt1, "i")) pt1 = pt2+2;
      if(pt1 == yk){
	puts("i@ȂBG[");	Er = '\0';
	while(! Er){
	  puts("̃L[ĉB");
	  Er = getchar();
	}
      }
      else {
	j = 1;	pt1 -= 2;
	while(yk != pt1--) ++j;
      }
      return(j);
    }
    return(j);
}

akitsume(){
    int  i, j, ct;
    for(i = 0; i <= wnb; i++)
      if(cj[i][0] == '\0') goto TUME;
    return;
    TUME:	ct = i-1;
    if(i == wnb){ wnb--;	return; }
    for(j = i+1; j <= wnb; j++)
      if(cj[j][0] != '\0'){ ct++;
	strcpy(cj[ct], cj[j]);
	strcpy(cz[ct], cz[j]);	strcpy(en[ct], en[j]);
      }
    wnb = ct;
    cj[wnb+1][0] = '\0';
/* printf("--- akitsume --- en[i-1] = %s en[i] = %s \n", en[i-1], en[i]);*/
}

transf(pt)
char *pt;
{
    int  i, lnb;
    char cmd[40], kaz[40], *pcm;
    i = *(pt-1)-'0';
    if(pcm = (char *)index(pt+2, ';')) ptcpy(cmd, pt+1, pcm);
    if(strcmp(cmd, "-i") == 0){
      kakudel(md[i]);	return;
    }
    lnb = strlen(cmd)-1;
    if(cmd[lnb] == '+'){
      cmd[lnb] = '\0';	ptinsert(md[i]+1, cmd);
    }
    if(strcmp(cmd, "RENYO") == 0){ renyo(i);	return; }
    if(strcmp(cmd, "TEONBI") == 0){ teonbi(md[i]);	return; }
    if(strcmp(cmd, "PAST") == 0){ past(md[i]);	return; }
    if(strcmp(cmd, "GENKE") == 0){ genkei(md[i]);	return; }
    if(strcmp(cmd, "NEGA") == 0){ nega(md[i]);	return; }
    if(strcmp(cmd, "GIMU") == 0){ gimu(md[i]);	return; }
    if(strcmp(cmd, "RARER") == 0){
      genkei(md[i]);	rareru(md[i]);	return;
    }
    if(strcmp(cmd, "SASER") == 0){ saser(md[i]);	return; }
    if(strcmp(cmd, "DES") == 0 && strcmp(cj[wc+i+1], "9̂ŁA")){
      desu(md[i]);	return;
    }
    if(strcmp(cmd, "DEKIR") == 0){ dekiru(md[i]);       return; }
    if(strcmp(cmd, "KANYU") == 0){ kanyuugobi(md[i]);	return; }
    if(strcmp(cmd, "SJ") == 0){ sopresv(sj, sz, i);	return; }
    if(strcmp(cmd, "OJ") == 0){ sopresv(oj, oz, i);	return; }
    if(strcmp(cmd, "VR") == 0){ vrpresv(i);	return; }
    if(cmd[0] == '-'){
      minus(cmd+1, *(pt-2), i);	return;
    }
    if(cmd[0] == '+'){ add(*(pt-2), i, cmd);	return; }
    if(strcmp(cmd, "Z") == 0){ strcpy(md[i], "Z");	return; }
    if(strcmp(cmd, "KAZ1") == 0){
      kaz[0] = '\0';	kazu1(i, kaz);	return;
    }
    if(strcmp(cmd, "KAZ2") == 0){ kazu2(i, kaz);	return; }
    if(strcmp(cmd, "K&K") == 0){ sei_mei(md[i], md[i+1]);	return; }
}

sopresv(bj, bz, i)
int  i;
char bj[], bz[];
{
    int  t, j, ln, lz, l3;
    char ctck[40], cz3[40], bjq[JSentLen], *ptj, c;
 if(Dbg == 'Y') fprintf(dbg, "--- soprev --- cz = %s \n", cz[wc+i]);
    strcpy(cz3, cz[wc+i]);
    if(*bz == 'b'){
      ptj = (char*)index(bz+1, 'b');
      strcpy(ctck, ptj);	strcat(ctck, "b");
    }
    else{
      if(*bj == '1'){
	*ctck = 'b';	*(ctck+1) = '\0';
	strcat(ctck, bz);	strcat(ctck, "b");
      }
      else *ctck = '\0';
    }
    lz = strlen(ctck)-2;
    l3 = strlen(cz3)-1;
    if(lz+l3 > 27){
      j = 0;	*bz = *ctck;
      for(t = 1; t <= lz; t++){
	if(! index(bz, ctck[t])) bz[++j] = ctck[t];
      }
      bz[++j] = 'b';	j = 0;	*ctck = *cz3;
      for(t = 1; t <= 29; t++) ctck[t] = '\0';
      for(t = 0; t <= l3; t++){
	if(! index(ctck, cz3[t])) ctck[++j] = cz3[t];
      }
      strcpy(cz[wc+i], ctck);
      if(strlen(bz) + strlen(ctck) > 30){
	printf("\n The length of bz excedes 30 !! \n");
//	printf(" p܂ [Y/N]= ");	c = '\0';
//	while(c != 'y' && c != 'n') c = tolower(getchar());
//        if(c != 'y') exit(1);
	return;
      }
    }
    else strcpy(bz, ctck);
    strcat(bz, cz[wc+i]);
    strcpy(bjq, md[i]);
    ln = strlen(bjq);
    if(ptj = (char*)strstr(bjq, "i")){
      ptptcut(bjq+1, ptj+1);	ln = strlen(bjq);
      bjq[ln-2] = '\0';	ln -= 2;	*bjq = '1';
    }
    if(ln > 13 && (ptj = (char*)strstr(bjq+3, "fƂ"))) *(ptj+2) = '\0';
    ptj = (char*)strstr(bjq, "");
    while(ptj && *(ptj+2) && (ptj != (char*)strstr(ptj, "̂"))
    && (ptj != (char*)strstr(ptj, "̕"))){
      ptptcut(bjq+1, ptj+1);	ptj = (char*)strstr(bjq, "");
    }
    ln = strlen(bjq);
    if((ptj = (char*)strstr(bjq, "j")) && strlen(ptj) >= 4)
      ptptcut(bjq+1, ptj+1);
    if(ln < JWordLen) strcpy(bj, bjq);
    else ptcpy(bj, bjq, bjq+WordLen-2);
}

vrpresv(i)
int  i;
{
    int  ln;
    char dums[100], mw[3], *pt;
    if(strlen(md[i]) > 100) return;;
    if(*vr == '\0'){
      ln = strlen(md[i]);
      if(ln > JWordLen-2){
	if(pt = (char*)strstr(md[i], "i")) ptcpy(vr, md[i]+1, pt);
	else ptcpy(vr, md[i]+1, md[i]+JWordLen-1);
      }
      else strcpy(vr, md[i]+1);
      strcpy(vz, cz[wc+i]);
    }
    else{
      ln = strlen(vr);
      left(dums, vr, ln-2);
      if(strcmp(dums, "")==0) strcpy(dums, vr);
      if(((! strstr(md[i], dums)) && strlen(md[i]) < 45) ||
      (strcmp(dums, "ł")==0 && (! index(cz[wc+i], 'v')))){
	if(pt = (char*)strstr(md[i], "A")) strcpy(vr, pt+2);
	else strcpy(vr, md[i]+1);
	strcpy(vz, cz[wc+i]);
      }
    }
    if(pt = (char*)index(vr, '>')) ptptcut(vr, pt);
    left(mw, vr, 2);
    if(strcmp(mw, "")==0 || strcmp(mw, "")==0 || strcmp(mw, "")==0 ||
    strcmp(mw, "")==0 || strcmp(mw, "")==0) ptcut(vr, 2);
    if(pt = (char*)strstr(vr, "̓")) ptcut(pt, 6);
    ln = strlen(vr);
    if(vr[ln-1] == ')') genkei(vr);
}

teonbi(md)
char md[];
{
    register int i;
    int  j, ln;
    char *gobi, pars[JWordLen];
    ln = strlen(md);
    j = paren(md);
    if(j > 2){
      if(md[ln-1] == ')') *pars = '\0';
      else{
	if(ln-j < JWordLen) strcpy(pars, md+j-1);
	else ptcpy(pars, md+j-1, md+j-2+JWordLen);
      }
      md[j-1] = '\0';	ln = j-1;
    }
    else *pars = '\0';
    gobi = md+ln-4;
    if(strcmp(gobi, "s") == 0){
      strcpy(gobi+2, "");	goto RTN;
    }
    if(strcmp(gobi, "") == 0){
      strcpy(gobi, "");	goto RTN;
    }
    if(strcmp(gobi, "") == 0){
      strcpy(gobi, "");	goto RTN;
    }
    if(strcmp(gobi, "") == 0){
      strcpy(gobi, "");	goto RTN;
    }
    if(strcmp(gobi, "Ȃ") == 0){ strcat(md, "");	goto RTN; }
    for(i = 0; i <= 31; i++)
      if(strcmp(gobi, ksm[i]) == 0){
	strcpy(gobi+2, "");	goto RTN;
      }
    gobi += 2;
    if(strcmp(gobi, "") == 0){ strcpy(gobi, "");	goto RTN; }
    if(strcmp(gobi, "") == 0 || strcmp(gobi, "") == 0
    || strcmp(gobi, "") == 0){
      strcpy(gobi, "");	goto RTN;
    }
    if(strcmp(gobi, "") == 0){ strcpy(gobi, "");	goto RTN; }
    for(i = 0; i <= 8; i++)
      if(strcmp(gobi, shs[i]) == 0){
	strcpy(gobi, onb[i]);	goto RTN;
      }
    RTN:	strcat(md, pars);
}

renyo(i)
int  i;
{
    int  j, ln;
    char *gobi, pars[JWordLen], *yk, *qt;
    yk = md[i];
    ln = strlen(yk);
    if(strcmp(yk+ln-2, ") ") == 0 && (qt = (char*)strstr(yk, " ("))){
      strcpy(pars, qt);
      *qt = '\0';
    }
    else *pars = '\0';
    ln = strlen(yk);
    if(ln >= 6){
      gobi = yk + ln -6;
      if(strcmp(gobi, "낤") == 0){
        *(yk + ln -6) = '\0';
        ln -= 6;
        if(ln >= 6) gobi -= 6;
      }
      if(strcmp(gobi, "") == 0){
        strcpy(gobi, "");
        goto RTN;
      }
      if(strcmp(gobi, "܂") == 0){
        *gobi = '\0';
        goto RTN;
      }
      if(strcmp(gobi, "Ȃ") == 0){
        strcpy(gobi+4, "");
        goto RTN;
      }
    }
    gobi = yk+ln-4;
    if(strcmp(gobi, "") == 0 ||strcmp(gobi, "") == 0 || strcmp(gobi, "") == 0){
      if (strncmp(gobi-2, "", 2) != 0){ strcpy(gobi+2, "");	goto RTN; }
    }
    if(strcmp(gobi, "") == 0){ strcpy(gobi, "");	goto RTN; }
    if(strcmp(gobi, "") == 0){ strcat(gobi, "悤");	goto RTN; }
    if(strcmp(gobi+2, "") == 0){ strcpy(gobi+2, "");	goto RTN; }
    if(strcmp(gobi+2, "") == 0 || strcmp(gobi+2, "") == 0){
      strcpy(gobi+2, "");	goto RTN;
    }
    for(j = 0; j <= 31; j++)
      if(strcmp(gobi, ksm[j]) == 0){ *(gobi+2) = '\0';	goto RTN; }
    gobi += 2;
    for(j = 0; j <= 8; j++)
      if(strcmp(gobi, shs[j]) == 0) strcpy(gobi, rny[j]);
    RTN:
    if(*pars) strcat(yk, pars);
    strcpy(md[i], yk);
}

nega(yk)
char yk[];
{
    int  j, ln;
    char pars[25], daro[7];
    ln = strlen(yk);
    if(yk[ln-1] != ')'){
      if(j = paren(yk)){
	strcpy(pars, yk+j-1);	yk[j-1] = '\0';
      }
      else *pars = '\0';
      strcpy(daro, yk+ln-6);
      if(strcmp(daro, "낤") == 0){
	*(yk+ln-6) = '\0';	ln -= 6;
      }
      else daro[0] = '\0';
      nega2(yk);	strcat(yk, daro);	strcat(yk, pars);
    }
    else {
      genkei(yk);	nega2(yk);	past(yk);
    }
}

nega2(yk)
char yk[];
{
    char *gobi;
    int  i, ln;
    ln = strlen(yk);	gobi = yk+ln-2;
    if(strcmp(gobi, "") == 0 || strcmp(gobi, "I") == 0){
      strcat(yk, "ł͂Ȃ");	return; 
    }
    if(strcmp(gobi, "") == 0){ strcpy(gobi, "ł͂Ȃ");	return; }
    if(ln >= 6){
      gobi = yk+ln-6;
      if(strcmp(gobi, "")== 0){ strcpy(gobi, "Ȃ( 3)");	return; }
      if(strcmp(gobi, "L")== 0){ strcpy(gobi, "( 3)");	return; }
      if(strcmp(gobi, "ł")== 0){ strcpy(gobi, "ł͂Ȃ");	return; }
    }
    gobi = yk+ln-4;
    if(strcmp(gobi, "Ȃ") == 0) return;
    if(strcmp(gobi, "") == 0){ strcpy(gobi, "Ȃ");	return; }
    if(strcmp(gobi, "L") == 0){ strcpy(gobi, "");	return; }
    if(strcmp(gobi, "") == 0){ strcpy(gobi, "Ȃ");	return; }
    if(strcmp(gobi, "") == 0){ strcpy(gobi, "Ȃ");	return; }
    if(strcmp(gobi, "܂") == 0){ strcpy(gobi, "܂");	return; }
    for(i = 0; i <= 31; i++)
      if(strcmp(gobi, ksm[i]) == 0){
	strcpy(gobi+2, "Ȃ");	return;
      }
    gobi += 2;
    if(strcmp(gobi, "") == 0){
      strcpy(gobi, "Ȃ( 3)");	return;
    }
    if(strcmp(gobi, "") == 0){
      strcpy(gobi, "Ȃ");	return;
    }
    for(i = 0; i <= 8; i++)
      if(strcmp(gobi, shs[i]) == 0){
	strcpy(gobi, mzn[i]);	strcat(yk, "Ȃ");	return;
      }
    //    strcat(yk, "̂ł͂Ȃ");
    strcat(yk, "Ƃ͂Ȃ");
}

gimu(mr)
char mr[];
{
    register int j;
    int  ln;
    char *gobi, pars[JWordLen];
    ln = strlen(mr);
    if(strcmp(mr+ln-2, ") ") == 0){
    }
    else *pars = '\0';
    ln = strlen(mr);	strcpy(pars, "Ȃ΂ȂȂ");
    if(ln >= 4){
      gobi = mr+ln-4;
      if(strcmp(gobi, "") == 0){
	strcpy(gobi, "");	goto RTN;
      }
      if(strcmp(gobi, "") == 0){
	strcpy(gobi, "");	goto RTN;
      }
      for(j = 0; j <= 31; j++)
	if(strcmp(gobi, ksm[j]) == 0){
	  strcpy(gobi+2, "");	goto RTN;
	}
    }
    gobi = mr+ln-2;
    if(strcmp(gobi, "") == 0){
      strcpy(gobi, "");	goto RTN;
    }
    for(j = 0; j <=8; j++)
      if(strcmp(gobi, shs[j]) == 0){
	strcpy(gobi, mzn[j]);	strcat(gobi+2, "");	goto RTN;
      }
    RTN:
    if(*pars) strcat(mr, pars);
}

rareru(md)
char md[];
{
    register int i;
    char *gobi;
    int  ln;
    ln = strlen(md);
    gobi = md+ln-2;
    if(ln > 5){
      gobi = md+ln-6;
      if(strcmp(gobi, "ł") == 0) return;
      if(strcmp(gobi, "") == 0) return;
      if(strcmp(gobi, "") == 0){ strcpy(gobi, "");	return; }
      if(strcmp(gobi, "") == 0){ strcpy(gobi, "");	return; }
    }
    gobi = md+ln-4;
    if(strcmp(gobi, "") == 0){
      strcpy(gobi, "");	return;
    }
    for(i = 0; i <= 31; i++){
      if(strcmp(gobi, ksm[i]) == 0){
	strcpy(gobi+2, "");	return;
      }
    }
    gobi += 2;
    for(i = 0; i <= 8; i++){
      if(strcmp(gobi, shs[i]) == 0){
	strcpy(gobi, mzn[i]);	strcat(gobi, "");	return;
      }
    }
}

saser(md)
char md[];
{
    int  i, sw, ln;
    char pars[JWordLen], *pt, *gobi;
    *pars = '\0';
    if(pt = (char*)strstr(md, "i")){
      strcpy(pars, pt);	*pt = '\0';
    }
    if(index(md, '(')) sw = 1;	else sw = 0;
    genkei(md);
 if(Dbg == 'Y') fprintf(dbg, "--- saser --- md = %s \n", md);
    ln = strlen(md);
    if(ln > 3){
      if(strcmp(md, "3") == 0){
	strcpy(md, "3点");	goto RTN;
      }
      gobi = md+ln-6;
      if(strcmp(gobi, "ł") == 0){
	strcpy(gobi, "ɂ");
	goto RTN;
      }
      gobi = md+ln-4;
      if(strcmp(gobi, "") == 0 || strcmp(gobi, "Ȃ") == 0){
	if(sw == 1) strcpy(gobi, "Ă炤");
	else strcpy(gobi, "");
	goto RTN;
      }
      for(i = 0; i <= 31; i++){
	if(strcmp(gobi,ksm[i]) == 0){
	  if(sw == 1) strcpy(gobi+2, "Ă炤");
	  else strcpy(gobi+2, "");
	  goto RTN;
        }
      }
    }
    gobi = md+ln-2;
    for(i = 0; i <= 8; i++){
      if(strcmp(gobi, shs[i]) == 0){
        strcpy(gobi, mzn[i]);
        if(sw == 1) strcat(md, "Ă炤");
        else strcat(md, "");
        break;
      }
    }
    RTN:	strcat(md, pars);
}

// Ɩ̏FJ^JiŎn܂ĂȂ΁A{lƂ݂ȂāAt]
sei_mei(n1, n2)
char n1[20], n2[20];
{
    unsigned char fst, fam, *pt;
    fst = n1[1];	fam = n2[1];
// printf("--- sei_mei ---  fst = %d  fam = %d \n", fst, fam);
    if((fst < 65 && fst != 32) || (fst > 90 && fst != 131) ||
       (fam < 65 && fam != 32) || (fam > 90 && fam != 131)) swap(n1, n2);
    else{
        if(isalpha(*(n1+1))){
  //      ptinsert(n1+1, " ");
          strcat(n1, " ");
        }
    }
    while(pt = (char*)strstr(n1, "  ")) ptcut(pt, 1);
    while(pt = (char*)strstr(n2, "  ")) ptcut(pt, 1);
}

desu(mr)
char mr[];
{
    int  i, ln, ln1;
    char z[3], k[18], *gobi;
    ln = strlen(mr);
    if(LOG == 'Y') fprintf(prcs, "mr = %s  \n", mr);
    gobi = mr + ln;
    strcpy(z, gobi-2);
    if(strcmp(z, "A") == 0){ *z = '\0';	goto RTN; }
    if(strcmp(z, "") == 0 || strcmp(z, "B") == 0){
      ptcut(gobi-2, 2);	ln -= 2;
    }
    else *z = '\0';
    gobi = mr+ln;
    for(i = 0; i <= 13; i++){
      ln1 = strlen(bungo[i]);
      if(ln > ln1-1 && strcmp(gobi-ln1, bungo[i]) == 0){
	strcpy(gobi-ln1, kougo[i]);	goto RTN;
      }
    }
    if(ln > 3 && strcmp(gobi-4, "") == 0){
      strcpy(gobi-4, "܂");	goto RTN;
    }
    if(ln > 12 && strcmp(gobi-12, "( 2)") == 0){
      strcpy(gobi-12, "܂");	goto RTN;
    }
    for(i = 0; i <= 31; i++){
      strcpy(k, ksm[i]);
      if(ln > 3 && strcmp(gobi-4, k) == 0){
        if(strcmp(gobi-6, "ׂ") == 0) break;
	strcpy(gobi-2, "܂");	goto RTN;
      }
      strcpy(k+2, "Ȃ");
      if(ln > 5 && strcmp(gobi-6, k) == 0){
        if(strncmp(k, "", 2) == 0) break;
	strcpy(gobi-4, "܂");	goto RTN;
      }
      strcpy(k+4, "( 3)");
      if(ln > 15 && strcmp(gobi-16, k) == 0){
	ptreplace(gobi-14, 14, "܂ł");	goto RTN;
      }
/*      ptreplace(k+2, 14, "( 1)");*/
      strcpy(k+2, "( 1)");
      if(ln > 9 && strcmp(gobi-10, k) == 0){
	strcpy(gobi-8, "܂");	goto RTN;
      }
    }
    for(i = 0; i <= 8; i++){
      if(ln > 1 && strcmp(gobi-2, shs[i]) == 0){
	strcpy(gobi-2, rny[i]);	strcat(mr, "܂");	goto RTN;
      }
      strcpy(k, mzn[i]);	strcat(k, "Ȃ");
      if(ln > 5 && strcmp(gobi-6, k) == 0){
	ptreplace(gobi-6, 6, rny[i]);	strcat(mr, "܂");	goto RTN;
      }
      strcpy(k+4, "( 3)");
      if(ln > 15 && strcmp(gobi-16, k) == 0){
	strcpy(gobi-16, rny[i]);	strcat(mr, "܂ł");	goto RTN;
      }
      *k = '\0';
      strink(k, "(", shs[i], " 2)", (char*)0);
      if(ln > 5 && strcmp(gobi-6, k) == 0){
	strcpy(gobi-10, rny[i]);	strcat(mr, "܂");	goto RTN;
      }
    }
    strcat(mr, "ł");
    RTN:	strcat(mr, z);
}

dekiru(char *md)
{
    int i, ln;
    char *gobi;
    ln = strlen(md);
    if(ln >= 8){
      gobi = md + ln -4;
      if(strcmp(gobi, "") == 0){
        strcpy(gobi, "ł");
        return;
      }
    }
    if(ln > 5){
      gobi = md + ln -6;
      if(strcmp(gobi, "") == 0){
        strcpy(gobi, "ł");
        return;
      }
      if(strcmp(gobi, "ł") == 0){
        strcpy(gobi, "ɂȂ");
        return;
      }
      gobi = md + ln -4;
      if(strcmp(gobi, "") == 0){
        strcpy(gobi, "ł");
        return;
      }
    }
    if(ln > 3){
      gobi = md + ln - 4;
      for(i = 0; i <= 31; i++){
        if(strcmp(gobi, ksm[i]) == 0){
          strcpy(gobi+2, "");
          return;
        }
      }
    }
    gobi = md + ln - 2;
    for(i = 0; i <= 8; i++){
      if(strcmp(gobi, shs[i]) == 0){
        strcpy(gobi, mei[i]);
        strcat(gobi, "");
        break;
      }
    }
  }

kanyuugobi(md)
char md[];
{
    int  i, ln;
    char *gobi;
    ln = strlen(md);
    if(ln > 3){
      gobi = md+ln-4;
      if(strcmp(gobi,"") == 0){ strcpy(gobi, "悤");	return; }
      for(i = 0; i <= 31; i++){
        if(strcmp(gobi, ksm[i]) == 0){
          strcpy(gobi+2, "悤");
          return;
        }
      }
    }
    gobi = md+ln-2;
    for(i = 0; i <= 8; i++){
      if(strcmp(gobi,shs[i]) == 0){
	strcpy(gobi, kanyu[i]);	return;
      }
    }
}

minus(str, c, i)
int  i;
char c, *str;
{
    char *pt, *pt1, *pt2;
    if(c == 'T')
      while(pt = (char*)strstr(cz[wc+i], str))
	ptcut(pt, 1);
    else {
      if(strcmp(str, "ij") == 0)
	while((pt1 = (char*)strstr(md[i], "i")) &&
	 (pt2 = (char*)strstr(pt1+2,"j"))){
	  pt = pt2+1;
	  do{ *(pt1++) = *(++pt);
	  } while(*pt);
	  return;
	}
      if(strcmp(str, "()") == 0)
	while((pt1 = (char*)index(md[i], '(')) &&
	 (pt2 = (char*)index(pt1+1,')'))){
	  ptptcut(pt1, pt2);
	}
      else
	while(pt = (char*) strstr(md[i], str)) ptcut(pt, strlen(str));
    }
}

add(c, i, str)
int  i;
char c, str[];
{
    char st1[50], *pt;
    strcpy(st1, str+1);
    if(strcmp(st1, "OZ$") == 0){
      if(pt = (char*)rindex(oz, 'y')) strcpy(st1, pt);
      else{
	if(pt = (char*)rindex(sz, 'y')) strcpy(st1, pt);
      }
      while(pt = (char *)index(st1, '?')) ptcut(pt, 1);
      strcat(cz[wc+i], st1);	return;
    }
    if(strcmp(st1, "SJ$") == 0) strcpy(st1, sj+1);
    if(strcmp(st1, "OJ$") == 0) strcpy(st1, oj+1);
    if(strcmp(st1, "VR$") == 0) strcpy(st1, vr);
    if(c == 'J') strcat(md[i], st1);
    else strcat(cz[wc+i], st1);
}

kazu1(i, kaz)
int  i;
char kaz[40];
{
    char bz[40], *pt, *pt2;
    strcpy(bz, cz[wc+i]);
    if(pt = (char*)index(bz, '%')) strcpy(pt, "");
    if(pt = (char*)rindex(bz, 'y')){
      strcpy(kaz, pt+1);
      if((pt2 = (char*)index(kaz, 'b')) && (pt = (char*)index(pt2, 'y')))
	ptptcut(kaz, pt);
    }
    else {
      if(index(bz, 'h')){
	strcpy(bz, "l");	return;
      }
      if(index(bz, 't')){
	kaz[0] = '\0';	return;
      }
    }
}

kazu2(i, kaz)
int  i;
char kaz[40];
{
    char bj[100];
    strcpy(bj, md[i]);
    kaz[2] = '\0';  // 15/9/30
    strcat(bj, kaz);
    //    if(kaz[0] != '\0') strcat(bj, "");
    strcpy(md[i],bj);
}

kakudel(str)
char str[];
{
    int  i;
    char jos[3];
    if(str[1] == '#'){
      ptcpy(jos, str+3, str+5);	i = 3;
    }
    else {
      ptcpy(jos, str+1, str+3);	i = 1;
    }
    if(strcmp(jos, "")==0 || strcmp(jos, "")==0 || strcmp(jos, "")==0
      || strcmp(jos, "") == 0
      || strcmp(jos, "")==0 && instr(str, "ł") != 2
       || strcmp(jos, "")==0 && instr(str, "ƂĂ") != 2  && strncmp(str+1, "Ƃ킯", 8) )
       ptcut(str+i, 2);
}

kakumv(st1, st2)
char st1[], st2[];
{
    char jos[3];
    ptcpy(jos, st1+1, st1+3);
    if(strcmp(jos, "")==0 || strcmp(jos, "")==0 || strcmp(jos, "")==0 ||
  (strcmp(jos, "")==0 && instr(st1, "ƂĂ")!=2 && instr(st1, "Ƃǂ")!=2)){
      ptcut(st1+1, 2);	ptinsert(st2+1, jos);
    }
}




